package travel

import (
	"path"

	"shylinux.com/x/ice"
	"shylinux.com/x/icebergs/base/cli"
	"shylinux.com/x/icebergs/base/mdb"
	"shylinux.com/x/icebergs/base/nfs"
	"shylinux.com/x/icebergs/base/tcp"
	"shylinux.com/x/icebergs/base/web"
	"shylinux.com/x/icebergs/core/code"
	kit "shylinux.com/x/toolkits"
	"shylinux.com/x/toolkits/task"
)

type Travel struct {
	ice.Space
	weight  string `name:"weight" help:"统计" icon:"bi bi-pie-chart"`
	foreach string `name:"foreach" help:"遍历" icon:"bi bi-card-list"`
	script  string `name:"create name*=2025-cluster port*=40000 count*=3" icon:"bi bi-terminal"`
	upgrade string `name:"upgrade space"`
	list    string `name:"list space auto" help:"集群管理"`
}

func (s Travel) Foreach(m *ice.Message, arg ...string) {
	m.Cmdy(s, s.Concurrent, ice.Info.Make.Module, m.PrefixKey()).PushAction(s.BinaryActionList(m)...).Action(s.Foreach, s.Upgrade, s.Publish)
}
func (s Travel) Script(m *ice.Message, arg ...string) {
	for i := 0; i < kit.Int(m.Option(mdb.COUNT)); i++ {
		m.Option(cli.CTX_ENV, cli.CTX_NAME, m.Option(mdb.NAME)+kit.Format(i))
		m.Option(cli.CTX_ARG, kit.JoinCmdArgs(ice.DEV, m.Option(ice.MSG_USERHOST), tcp.NODENAME, m.Option(mdb.NAME)+kit.Format(i), tcp.PORT, kit.Format(kit.Int(m.Option(tcp.PORT))+i)))
		m.Cmdy(code.PUBLISH, nfs.BINARY)
	}
	m.Action(s.Foreach, s.Script)
}
func (s Travel) Publish(m *ice.Message, arg ...string) {
	m.Cmdy(web.SPACE, ice.OPS, web.DREAM, code.PUBLISH, m.Option(ice.MSG_USERPOD))
}
func (s Travel) Upgrade(m *ice.Message, arg ...string) {
	if m.Option(web.SPACE) != "" {
		m.Cmd(web.SPACE, m.Option(web.SPACE), code.UPGRADE)
		return
	}
	m.Cmdy(s, s.Sequential, ice.Info.Make.Module, code.UPGRADE).Action(s.Foreach)
}
func (s Travel) List(m *ice.Message, arg ...string) {
	if len(arg) == 0 {
		stat := map[string]int{}
		m.SpaceListServer().Table(func(value ice.Maps) { stat[value[nfs.MODULE]]++ })
		kit.For([]string{
			"2025-dev.shylinux.com/x/20250213-machine",
			"2025-dev.shylinux.com/x/20250211-service",
		}, func(key string) {
			m.Push(path.Base(key), stat[key])
		})
		m.Push(nfs.VERSION, ice.Info.Make.Version)
		m.Push(mdb.TIME, ice.Info.Make.Time)
		m.Cmdy("", s.Script)
	} else {
		m.Cmdy(web.SPACE, arg[0], m.PrefixKey())
	}
}

func init() { ice.Cmd("web.chat.dev.cluster.travel", Travel{}) }

func (s Travel) Concurrent(m *ice.Message, arg ...string) {
	m.SpaceListServer().TableGo(func(value ice.Maps, lock *task.Lock) { s.pushRecord(m, value, arg...) })
	m.SortStrR(web.SPACE)
}
func (s Travel) Sequential(m *ice.Message, arg ...string) {
	m.SpaceListServer().Table(func(value ice.Maps) { s.pushRecord(m, value, arg...) })
	m.SortStrR(web.SPACE)
}
func (s Travel) pushRecord(m *ice.Message, value ice.Maps, arg ...string) {
	if value[nfs.MODULE] == arg[0] {
		m.Cmd(web.SPACE, value[mdb.NAME], arg[1:]).Table(func(val ice.Maps, index int, head []string) {
			m.Push(web.SPACE, value[mdb.NAME]).PushRecord(val, head...)
		})
	}
	if value[nfs.MODULE] == "2025-dev.shylinux.com/x/20250215-cluster" {
		m.Copy(m.Cmd(web.SPACE, value[mdb.NAME], m.PrefixKey(), m.ActionKey(), arg).RewriteAppend(func(val, key string, index int) string {
			kit.If(key == web.SPACE, func() { val = kit.Keys(value[mdb.NAME], val) })
			return val
		}))
	}
}
